@@ -242,7 +242,7 @@ static void add_to_stats(ogg_parser_data *opd,kate_stream *ks,ogg_packet *op)
   }
 }
 
-static int ogg_parser_on_page(kate_uintptr_t data,ogg_page *og)
+static int ogg_parser_on_page(kate_uintptr_t data,long offset,ogg_page *og)
 {
   ogg_parser_data *opd=(ogg_parser_data*)data;
   ogg_packet op;
@@ -261,8 +261,8 @@ static int ogg_parser_on_page(kate_uintptr_t data,ogg_page *og)
   }
 
   if (ks) {
-    kprintf(NULL,klt_container,"Ogg page, granpos %016llx, %d bytes, %d packets on this page\n",
-        (long long)ogg_page_granulepos(og),
+    kprintf(NULL,klt_container,"Ogg page at offset %lx, granpos %016llx, %d bytes, %d packets on this page\n",
+        offset,(long long)ogg_page_granulepos(og),
         og->header_len+og->body_len,ogg_page_packets(og));
 
     katalyzer_dump_data(ks,klt_dump,"Page header",og->header,og->header_len);
@@ -271,6 +271,7 @@ static int ogg_parser_on_page(kate_uintptr_t data,ogg_page *og)
     while ((ret=ogg_stream_packetout(&ks->os,&op))) {
       if (ret<0) {
         kprintf(ks,klt_error,"Hole in data\n");
+        continue;
       }
       is_kate=(ks->init>=kstream_header_info);
       if (!is_kate) {
@@ -352,6 +353,13 @@ static int ogg_parser_on_page(kate_uintptr_t data,ogg_page *og)
   return 0;
 }
 
+static int ogg_parser_on_hole(kate_uintptr_t data,long offset,long size)
+{
+  (void)data;
+  kprintf(NULL,klt_error,"Hole in data at offset %lx: %ld bytes\n",offset,size);
+  return 0;
+}
+
 static void katalyzer_output_stats(const katalyzer_stats *stats)
 {
   const kate_stream *ks=stats->ks;
@@ -482,6 +490,23 @@ static void print_version(void)
   printf("Katalyzer, a Kate stream analyzer - %s\n",kate_get_version_string());
 }
 
+static void print_help(const char *argv0)
+{
+  size_t n;
+
+  printf("usage: %s [options] [filename]\n",argv0);
+  printf("   -V                  version\n");
+  printf("   -h                  help\n");
+  printf("   -l <types>          select information to log\n");
+  printf("   -l!                 log everything\n");
+  for (n=0;n<sizeof(log_types_info)/sizeof(log_types_info[0]);++n) {
+    printf("       %c        %s (default %s)\n",
+        log_types_info[n].id,
+        log_types_info[n].desc,
+        log_types_info[n].by_default?"on":"off");
+  }
+}
+
 int main(int argc,char **argv)
 {
   size_t bytes_read;
@@ -497,27 +522,30 @@ int main(int argc,char **argv)
   int headers_written=0;
   ogg_parser_data opd;
   ogg_parser_funcs opf;
-  size_t n;
 
   for (arg=1;arg<argc;++arg) {
     if (argv[arg][0]=='-') {
       switch (argv[arg][1]) {
+        case '-':
+          if (!strcmp(argv[arg],"--version")) {
+            print_version();
+            exit(0);
+          }
+          else if (!strcmp(argv[arg],"--help")) {
+            print_help(argv[0]);
+            exit(0);
+          }
+          else {
+            fprintf(stderr,"Invalid option: %s\n",argv[arg]);
+            exit(-1);
+          }
+          break;
         case 'V':
           print_version();
           exit(0);
         case 'h':
           print_version();
-          printf("usage: %s [options] [filename]\n",argv[0]);
-          printf("   -V                  version\n");
-          printf("   -h                  help\n");
-          printf("   -l <types>          select information to log\n");
-          printf("   -l!                 log everything\n");
-          for (n=0;n<sizeof(log_types_info)/sizeof(log_types_info[0]);++n) {
-            printf("       %c        %s (default %s)\n",
-                log_types_info[n].id,
-                log_types_info[n].desc,
-                log_types_info[n].by_default?"on":"off");
-          }
+          print_help(argv[0]);
           exit(0);
         case 'l':
           switch (argv[arg][2]) {
@@ -530,8 +558,8 @@ int main(int argc,char **argv)
               add_log_types(&log_types,new_log_types);
               break;
             default:
-              fprintf(stderr,"Invalid log type modifier: %s\n",argv[arg]);
-              exit(-1);
+              add_log_types(&log_types,argv[arg]+2);
+              break;
           }
           break;
         default:
@@ -623,6 +651,7 @@ int main(int argc,char **argv)
     init_ogg_parser_data(&opd);
 
     opf.on_page=&ogg_parser_on_page;
+    opf.on_hole=&ogg_parser_on_hole;
     ret=parse_ogg_stream(fin,signature,signature_size,opf,(kate_uintptr_t)&opd);
 
     output_all_stats(&opd);
