@@ -41,29 +41,34 @@ typedef struct {
 
   const char *output_filename;
   int verbose;
-  int event_index;
+#ifdef DEBUG
   int fuzz;
   unsigned long fuzz_seed;
+#endif
 
   void (*write_start_function)(FILE*);
   void (*write_end_function)(FILE*);
   void (*write_headers_function)(FILE*,const kate_info*,const kate_comment*);
-  void (*write_event_function)(FILE*,void*,const kate_event*,ogg_int64_t,int);
-  void *write_event_function_data;
-  struct lrc_data lrc_data;
+  void (*write_event_function)(FILE*,void*,const kate_event*,ogg_int64_t);
+  void *(*new_event_function_data)(void);
+  void (*free_event_function_data)(void*);
 } ogg_parser_data;
 
-static int ogg_parser_on_page(kate_uintptr_t data,ogg_page *og)
+static int ogg_parser_on_page(kate_uintptr_t data,long offset,ogg_page *og)
 {
   ogg_parser_data *opd=(ogg_parser_data*)data;
   ogg_packet op;
   int ret;
   kate_stream *ks;
 
+  /* check for hole */
+  if (!og) return 0;
+
   /* add the page in the correct stream */
   if (ogg_page_bos(og)) {
     add_kate_stream(&opd->kate_streams,og,opd->n_streams++);
     ks=find_kate_stream_for_page(&opd->kate_streams,og);
+    if (opd->new_event_function_data) ks->data=(*opd->new_event_function_data)();
     if (opd->verbose) {
       printf("found BOS page, serial %08x\n",ogg_page_serialno(og));
     }
@@ -172,7 +177,7 @@ static int ogg_parser_on_page(kate_uintptr_t data,ogg_page *og)
           }
           else if (ret==0) {
             if (opd->write_event_function) {
-              (*opd->write_event_function)(ks->fout,&opd->lrc_data,ev,ogg_page_granulepos(og),opd->event_index++);
+              (*opd->write_event_function)(ks->fout,ks->data,ev,ogg_page_granulepos(og));
             }
           }
         }
@@ -188,6 +193,20 @@ static void print_version(void)
   printf("Kate reference decoder - %s\n",kate_get_version_string());
 }
 
+static void print_help(const char *argv0)
+{
+  printf("usage: %s [options] [filename]\n",argv0);
+  printf("   -V                  version\n");
+  printf("   -v                  verbose\n");
+  printf("   -h                  help\n");
+  printf("   -o <filename>       set output filename\n");
+  printf("   -t <type>           set output format (kate (default), srt, lrc)\n");
+#ifdef DEBUG
+  printf("   -B                  write some bitmaps in /tmp (debug)\n");
+  printf("   -f <number>         fuzz testing with given seed (debug)\n");
+#endif
+}
+
 int main(int argc,char **argv)
 {
   size_t bytes_read;
@@ -203,8 +222,10 @@ int main(int argc,char **argv)
   char *buffer=NULL;
   ogg_int64_t bytes;
   int headers_written=0;
+#ifdef DEBUG
   int fuzz=0;
   unsigned long fuzz_seed=0;
+#endif
   ogg_parser_data opd;
   ogg_parser_funcs opf;
   size_t n;
@@ -212,19 +233,25 @@ int main(int argc,char **argv)
   for (arg=1;arg<argc;++arg) {
     if (argv[arg][0]=='-') {
       switch (argv[arg][1]) {
+        case '-':
+          if (!strcmp(argv[arg],"--version")) {
+            print_version();
+            exit(0);
+          }
+          else if (!strcmp(argv[arg],"--help")) {
+            print_help(argv[0]);
+            exit(0);
+          }
+          else {
+            fprintf(stderr,"Invalid option: %s\n",argv[arg]);
+            exit(-1);
+          }
+          break;
         case 'V':
           print_version();
           exit(0);
         case 'h':
-          print_version();
-          printf("usage: %s [options] [filename]\n",argv[0]);
-          printf("   -V                  version\n");
-          printf("   -v                  verbose\n");
-          printf("   -h                  help\n");
-          printf("   -o <filename>       set output filename\n");
-          printf("   -t <type>           set output format (kate (default), srt, lrc)\n");
-          printf("   -B                  write some bitmaps in /tmp (debug)\n");
-          printf("   -f <number>         fuzz testing with given seed (debug)\n");
+          print_help(argv[0]);
           exit(0);
         case 'o':
           if (!output_filename) {
@@ -238,13 +265,6 @@ int main(int argc,char **argv)
         case 'v':
           ++verbose;
           break;
-        case 'B':
-          write_bitmaps=1;
-          break;
-        case 'f':
-          fuzz=1;
-          fuzz_seed=strtoul(eat_arg(argc,argv,&arg),NULL,10);
-          break;
         case 't':
           if (!output_filename_type) {
             output_filename_type=eat_arg(argc,argv,&arg);
@@ -254,6 +274,15 @@ int main(int argc,char **argv)
             exit(-1);
           }
           break;
+#ifdef DEBUG
+        case 'B':
+          write_bitmaps=1;
+          break;
+        case 'f':
+          fuzz=1;
+          fuzz_seed=strtoul(eat_arg(argc,argv,&arg),NULL,10);
+          break;
+#endif
         default:
           fprintf(stderr,"Invalid option: %s\n",argv[arg]);
           exit(-1);
@@ -275,21 +304,24 @@ int main(int argc,char **argv)
     opd.write_headers_function=&write_kate_headers;
     opd.write_end_function=&write_kate_end;
     opd.write_event_function=&write_kate_event;
-    opd.write_event_function_data=NULL;
+    opd.new_event_function_data=NULL;
+    opd.free_event_function_data=NULL;
   }
   else if (!strcmp(output_filename_type,"srt")) {
     opd.write_start_function=NULL;
     opd.write_headers_function=NULL;
     opd.write_end_function=NULL;
     opd.write_event_function=&write_srt_event;
-    opd.write_event_function_data=NULL;
+    opd.new_event_function_data=&new_srt_data;
+    opd.free_event_function_data=&free_srt_data;
   }
   else if (!strcmp(output_filename_type,"lrc")) {
     opd.write_start_function=NULL;
     opd.write_headers_function=NULL;
     opd.write_end_function=&write_lrc_end;
     opd.write_event_function=&write_lrc_event;
-    opd.lrc_data.last_event_end_time=(kate_float)-1.0;
+    opd.new_event_function_data=&new_lrc_data;
+    opd.free_event_function_data=&free_lrc_data;
   }
   else {
     fprintf(stderr,"Invalid format type: %s\n",output_filename_type);
@@ -312,8 +344,8 @@ int main(int argc,char **argv)
     /* raw Kate stream */
     kate_state k;
     FILE *fout;
-    int event_index=0;
     int ret;
+    void *write_event_function_data=(*opd.new_event_function_data)();
 
     bytes=64;
     buffer=(char*)kate_malloc(bytes);
@@ -346,7 +378,9 @@ int main(int argc,char **argv)
       const kate_event *ev=NULL;
       kate_packet kp;
       kate_packet_wrap(&kp,bytes,buffer);
+#ifdef DEBUG
       if (fuzz) fuzz_kate_packet(&fuzz_seed,&kp);
+#endif
       ret=kate_high_decode_packetin(&k,&kp,&ev);
       if (ret<0) {
         fprintf(stderr,"failed to decode raw kate packet (%d)\n",ret);
@@ -363,9 +397,8 @@ int main(int argc,char **argv)
       }
       if (ev) {
         if (opd.write_event_function) {
-          (*opd.write_event_function)(fout,opd.write_event_function_data,ev,-1,event_index);
+          (*opd.write_event_function)(fout,write_event_function_data,ev,-1);
         }
-        ++event_index;
       }
 
       /* all subsequent packets are prefixed with 64 bits (signed) of the packet length in bytes */
@@ -380,6 +413,8 @@ int main(int argc,char **argv)
     }
     kate_free(buffer);
 
+    (*opd.free_event_function_data)(write_event_function_data);
+
     if (fout!=stdout) fclose(fout);
   }
   else {
@@ -388,12 +423,14 @@ int main(int argc,char **argv)
     init_kate_stream_set(&opd.kate_streams);
     opd.n_streams=0;
     opd.verbose=verbose;
-    opd.event_index=0;
+#ifdef DEBUG
     opd.fuzz=fuzz;
     opd.fuzz_seed=fuzz_seed;
+#endif
     opd.output_filename=output_filename;
 
     opf.on_page=&ogg_parser_on_page;
+    opf.on_hole=NULL;
     ret=parse_ogg_stream(fin,signature,signature_size,opf,(kate_uintptr_t)&opd);
 
     /* Force EOS on truncated streams */
@@ -405,6 +442,12 @@ int main(int argc,char **argv)
       }
     }
 
+    /* Clear user data */
+    for (n=0;n<opd.kate_streams.n_kate_streams;++n) {
+      kate_stream *ks=opd.kate_streams.kate_streams[n];
+      if (opd.free_event_function_data) (*opd.free_event_function_data)(ks->data);
+    }
+
     clear_kate_stream_set(&opd.kate_streams);
   }
 
