@@ -50,7 +50,9 @@ char base_path[4096]="";
 
 static char str[4096];
 static int headers_written=0;
+#ifdef DEBUG
 static int raw=0;
+#endif
 static kate_float repeat_threshold=(kate_float)0;
 static kate_float keepalive_threshold=(kate_float)0;
 static kate_float last_stream_time=(kate_float)0;
@@ -79,6 +81,7 @@ static int flush_page(FILE *f)
       fprintf(stderr,"Write failed (%s)\n",strerror(errno));
       return -1;
     }
+    fflush(f);
   }
   return 0;
 }
@@ -114,6 +117,7 @@ int write_headers(FILE *f)
     }
     if (ret>0) break; /* we're done */
 
+#ifdef DEBUG
     if (raw) {
       int ret=send_packet(f,&op,(kate_float)-1);
       if (ret<0) {
@@ -121,7 +125,9 @@ int write_headers(FILE *f)
         return ret;
       }
     }
-    else {
+    else
+#endif
+    {
       ogg_stream_packetin(&os,&op);
       ogg_packet_clear(&op);
     }
@@ -140,6 +146,7 @@ int send_packet(FILE *f,ogg_packet *op,kate_float t)
   if (t>last_stream_time)
     last_stream_time=t;
 
+#ifdef DEBUG
   if (raw) {
     if (op->packetno>0) {
       ogg_int64_t bytes=op->bytes;
@@ -154,9 +161,12 @@ int send_packet(FILE *f,ogg_packet *op,kate_float t)
       fprintf(stderr,"Write failed (%s)\n",strerror(errno));
       return -1;
     }
+    fflush(f);
     return 0;
   }
-  else {
+  else
+#endif
+  {
     ogg_stream_packetin(&os,op);
     ogg_packet_clear(op);
     return poll_page(f);
@@ -720,6 +730,26 @@ static void print_version(void)
   printf("Kate reference encoder - %s\n",kate_get_version_string());
 }
 
+static void print_help(const char *argv0)
+{
+  printf("usage: %s [options] [filename]\n",argv0);
+  printf("   -V                  version\n");
+  printf("   -h                  help\n");
+  printf("   -o <filename>       set output filename\n");
+  printf("   -t <type>           set input file type\n");
+  printf("       types can be: kate, srt, lrc\n");
+  printf("   -l <language>       set stream language\n");
+  printf("   -c <category>       set stream category\n");
+  printf("   -s <hex number>     set serial number of output stream\n");
+  printf("   -R <threshold>      Use repeat packets with given threshold (seconds)\n");
+  printf("   -K <threshold>      Use keepalive packets with given threshold (seconds)\n");
+  printf("   -C <tag>=<value>    Add comment to the Kate stream\n");
+  printf("   -M                  Allow simple markup in SRT files\n");
+#ifdef DEBUG
+  printf("   -r                  write raw Kate stream (experimental)\n");
+#endif
+}
+
 #ifdef DEBUG
 static void print_rle_stats(void)
 {
@@ -744,6 +774,26 @@ static void print_rle_stats(void)
 }
 #endif
 
+static void set_encoder_comment(kate_comment *kc)
+{
+  int ret;
+  const char *base="kateenc - ";
+  const char *version=kate_get_version_string();
+  size_t len=strlen(base)+strlen(version)+1;
+  char *value=(char*)kate_malloc(len);
+  if (!value) {
+    fprintf(stderr,"Failed to allocate %zu bytes\n",len);
+    exit(-1);
+  }
+  sprintf(value,"%s%s",base,version);
+  ret=kate_comment_add_tag(kc,"ENCODER",value);
+  kate_free(value);
+  if (ret<0) {
+    fprintf(stderr,"kate_comment_add_tag failed: %d\n",ret);
+    exit(-1);
+  }
+}
+
 int main(int argc,char **argv)
 {
   int n,ret,failed=0;
@@ -767,6 +817,11 @@ int main(int argc,char **argv)
     fprintf(stderr,"kate_comment_init failed: %d\n",ret);
     exit(-1);
   }
+  set_encoder_comment(&kc);
+  if (ret<0) {
+    fprintf(stderr,"Failed to set encoder tag: %d\n",ret);
+    exit(-1);
+  }
   ret=kate_info_init(&ki);
   if (ret<0) {
     fprintf(stderr,"kate_comment_init failed: %d\n",ret);
@@ -776,25 +831,25 @@ int main(int argc,char **argv)
   for (n=1;n<argc;++n) {
     if (argv[n][0]=='-') {
       switch (argv[n][1]) {
+        case '-':
+          if (!strcmp(argv[n],"--version")) {
+            print_version();
+            exit(0);
+          }
+          else if (!strcmp(argv[n],"--help")) {
+            print_help(argv[0]);
+            exit(0);
+          }
+          else {
+            fprintf(stderr,"Invalid option: %s\n",argv[n]);
+            exit(-1);
+          }
+          break;
         case 'V':
           print_version();
           exit(0);
         case 'h':
-          print_version();
-          printf("usage: %s [options] [filename]\n",argv[0]);
-          printf("   -V                  version\n");
-          printf("   -h                  help\n");
-          printf("   -o <filename>       set output filename\n");
-          printf("   -t <type>           set input file type\n");
-          printf("       types can be: kate, srt, lrc\n");
-          printf("   -l <language>       set stream language\n");
-          printf("   -c <category>       set stream category\n");
-          printf("   -s <hex number>     set serial number of output stream\n");
-          printf("   -r                  write raw Kate stream (experimental)\n");
-          printf("   -R <threshold>      Use repeat packets with given threshold (seconds)\n");
-          printf("   -K <threshold>      Use keepalive packets with given threshold (seconds)\n");
-          printf("   -C <tag>=<value>    Add comment to the Kate stream\n");
-          printf("   -M                  Allow simple markup in SRT files\n");
+          print_help(argv[0]);
           exit(0);
         case 'o':
           if (!output_filename) {
@@ -835,9 +890,6 @@ int main(int argc,char **argv)
         case 's':
           serial=strtoul(eat_arg(argc,argv,&n),NULL,16);
           break;
-        case 'r':
-          raw=1;
-          break;
         case 'C':
           comment=eat_arg(argc,argv,&n);
           /* check there's a = sign though */
@@ -872,6 +924,11 @@ int main(int argc,char **argv)
         case 'M':
           allow_srt_markup=1;
           break;
+#ifdef DEBUG
+        case 'r':
+          raw=1;
+          break;
+#endif
         default:
           fprintf(stderr,"Invalid option: %s\n",argv[n]);
           exit(-1);
@@ -949,7 +1006,10 @@ int main(int argc,char **argv)
     exit(-1);
   }
 
-  if (!raw) ogg_stream_init(&os,serial);
+#ifdef DEBUG
+  if (!raw)
+#endif
+    ogg_stream_init(&os,serial);
 
   ret=0;
   if (!strcmp(input_file_type,"kate")) {
@@ -981,7 +1041,10 @@ int main(int argc,char **argv)
         failed=ret;
       }
       else {
-        if (!raw) {
+#ifdef DEBUG
+        if (!raw)
+#endif
+        {
           ret=flush_page(fout);
           if (ret<0) {
             fprintf(stderr,"error flushing page: %d\n",ret);
@@ -992,7 +1055,11 @@ int main(int argc,char **argv)
     }
   }
 
-  if (!raw) ogg_stream_clear(&os);
+#ifdef DEBUG
+  if (!raw)
+#endif
+    ogg_stream_clear(&os);
+
   ret=kate_clear(&k);
   if (ret<0) {
     fprintf(stderr,"kate_clear failed: %d\n",ret);
